#!/bin/sh
# Federico Bareilles
# 13/12/2005
#
# Simple interfece for Star I on dialog.

VERSION=0.2.1
TITLE="Star I script utility v$VERSION"
CONFFILE=$HOME/.stari.conf
USTARICONF=$HOME/.ustari.conf
LOCK_EXT=.lock
#USTARIPATH=$HOME/tmp
USTARIPATH=/tmp
if [ -x /sbin/pidof ]; then
    PIDOF=/sbin/pidof
elif [ -x /bin/pidof ]; then
    PIDOF=/bin/pidof
else
    PIDOF=pidof
fi
MYPID=$($PIDOF -xs $0)
test -d $USTARIPATH || mkdir $USTARIPATH
: ${DIALOG=dialog}
tempfile=`tempfile 2>/dev/null` || tempfile=$USTARIPATH/tmp-$$
resultfile=`resultfile 2>/dev/null` || resultfile=$USTARIPATH/result-$$
trap "rm -f $tempfile $resultfile" 0 1 2 5 15
CLEAR="--clear"
#CLEAR=""
AUTODISP=off
DELAY=0
LASTBASENAME=""
#FILTER=$(grep "^FILTER" $CONFFILE |tail -n 1|\
#	 sed "s/^FILTER[ ]*=[ ]*\([a-zA-Z]\).*/\1/")
#FILTER=$(filter)
# FITS extension:
FITS=fits

if [ -f $USTARICONF ]; then
    . $USTARICONF
fi

export ETIME GAIN AUTODISP FILTER DELAY LASTBASENAME


save_ustariconf(){
(cat<<EOF
# $TITLE
# Federico A. Bareilles
# $(date +%d/%m/%Y)
#
AUTODISP=$AUTODISP
DELAY=$DELAY
LASTBASENAME=$LASTBASENAME
# FITS extension:
FITS=$FITS
EOF
) > $USTARICONF
}


set_etime_menu(){
    $DIALOG --backtitle "$TITLE" \
	--title "Set exposure time" $CLEAR \
        --inputbox "Enter new exposure time in secons\n
    " 8 51 $ETIME 2> $tempfile
    retval=$?

    case $retval in
    0)
	etime $(cat $tempfile) 2> $resultfile
	;;    
    1)
	echo "Cancel pressed." > $resultfile;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	;;
    esac
}


set_delay_menu(){
    $DIALOG --backtitle "$TITLE" \
	--title "Set delay time" $CLEAR \
        --inputbox "Enter new delay time in secons\n
    " 8 51 $DELAY 2> $tempfile
    retval=$?

    case $retval in
    0)
	if [ ! "$(cat $tempfile | grep  -ve [a-zA-Z\.])" == "" ]; then
	    DELAY=$(cat $tempfile) 2> $resultfile
	else
	    echo "Bad Argument." > $resultfile
	fi
	;;    
    1)
	echo "Cancel pressed." > $resultfile;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	;;
    esac
}



gain_is(){
    if [ "$GAIN" == "$1" ]; then
	echo "on"
    else
	echo "off"
    fi
}

set_gain_menu(){
    $DIALOG --backtitle "$TITLE" \
	--title "Set gain factor" $CLEAR \
        --radiolist "Select gain factor:\n Use [space] for select another gain\n\n" 11 51 2 \
	"1"  " Gain factor is 50.0 elec/adu " $(gain_is 1) \
        "4"    " Gain factor is 12.5 elec/adu " $(gain_is 4) \
    2> $tempfile
    retval=$?

    case $retval in
    0)
	gain $(cat $tempfile) 2> $resultfile
	;;    
    1)
	echo "Cancel pressed." > $resultfile;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	;;
    esac
}

filter_is(){
    if [ "$FILTER" == "$1" ]; then
	echo "on"
    else
	echo "off"
    fi
}

set_filter_menu(){
    # N V B R I

    $DIALOG --backtitle "$TITLE" \
	--title "Set filter" $CLEAR \
        --radiolist "Select filter for header information:\n Use [space] for select another filter\n\n" 14 51 5 \
	"N" "None"     $(filter_is N) \
	"B" "Blue"     $(filter_is B) \
	"V" "Visual"   $(filter_is V) \
	"R" "Red"      $(filter_is R) \
	"I" "Infrared" $(filter_is I) \
    2> $tempfile
    retval=$?

    case $retval in
    0)
	FILTER=$(cat $tempfile)
	filter $FILTER
	;;    
    1)
	echo "Cancel pressed." > $resultfile;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	;;
    esac
}



sequence_menu(){
    LTITLE=$3
    BASENAME=$2
    CMD=$1

    if [ "$BASENAME" == "" ];then
	$DIALOG --backtitle "$TITLE" \
	    --title "$LTITLE" $CLEAR \
	    --inputbox "Enter new base name for files\n
	    " 8 51 $LASTBASENAME 2> $tempfile
	retval=$?

	case $retval in
	0)
	    BASENAME=$(cat $tempfile) 2> $resultfile
	    LASTBASENAME=$BASENAME
	    ;;    
	1)
	    echo "Cancel pressed." > $resultfile
	    return ;;
	255)
	    if test -s $tempfile ; then
		cat $tempfile > $resultfile
	    else
		echo "ESC pressed." > $resultfile  
	    fi
	    return ;;
	esac
    fi

    gain $GAIM
    $DIALOG --backtitle "$TITLE" \
	--title "$LTITLE" $CLEAR \
        --inputbox "Enter number of images:\n
    " 8 51 2> $tempfile
    retval=$?

    case $retval in
    0)
	NUMSEQ=$(cat $tempfile| grep "[0-9]*"|grep -v [a-zA-Z.])
	if [ "$NUMSEQ" == "" ]; then
	    echo "Bad Number." > $resultfile
	    return
	fi
	;;    
    1)
	echo "Cancel pressed." > $resultfile
	return ;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	return;;
    esac

    OFFSET=$(ls -1 $BASENAME*.$FITS|tail -n 1|sed "s/$BASENAME[0]*\([0-9]*\).$FITS/\1/") 2>/dev/null
    let "OFFSET = (($OFFSET + 1))"
    $DIALOG --backtitle "$TITLE" \
	--title "$LTITLE" $CLEAR \
        --inputbox "First images is number:\n
    " 8 51 $OFFSET 2> $tempfile
    retval=$?

    case $retval in
    0)
	if [ "$(cat $tempfile| grep "[0-9]*"|grep -v [a-zA-Z.])" == "" ]; then
	    echo "Bad Number." > $resultfile
	    return
	fi
	let "OFFSET = (($(cat $tempfile) - 1))"
	;;    
    1)
	echo "Cancel pressed." > $resultfile
	return ;;
    255)
	if test -s $tempfile ; then
	    cat $tempfile > $resultfile
	else
	    echo "ESC pressed." > $resultfile  
	fi
	return;;
    esac

    echo ""> $resultfile
    filter $FILTER
    info="$(etime $ETIME)\n"

    ((for n in $(seq $NUMSEQ) ; do
	let "ind = (($n + $OFFSET))"
	name=$(printf "%s%03d" $BASENAME $ind)
	if [ "$CMD" != "bias" ]; then
	    echo "XXX"
	    echo -e "$info"
	    echo "exposing ($ETIME sec.)..."
	    echo "XXX"
	fi
	$CMD
	echo "XXX"
	echo "getting [$n/$NUMSEQ]: $name"
	#get $name >/dev/null
	if [ -f $name.$FITS ]; then 
	    echo "Overwriting $name.$FITS"
	fi
    	echo "XXX"
	info=$(get $name)
	if [ "$AUTODISP" == "on" ];then
	    xpaset -p ds9 file $PWD/$name.$FITS
	fi
    	printf "%.0f\n" $(echo "$n / $NUMSEQ * 100" | bc -l)
	
	if [ "$DELAY" != "0" ]; then
	    if [ "$n" != "$NUMSEQ" ]; then
		for s in $(seq $DELAY);do
		    #let "p=(($s * 100 / $DELAY))"
		    echo "XXX"
		    echo -e "$info"
		    echo "sleeping: $s/$DELAY, file $n/$NUMSEQ"
		    echo "XXX"
		    sleep 1
		done
	    fi
	fi

    done &)|\
    $DIALOG  --stdout --backtitle "$TITLE" --title "$LTITLE" \
	    --gauge "" 8 50 0)  |head -n 1 > $resultfile
#    $DIALOG --backtitle "$TITLE" --title "$LTITLE" \
#	    --gauge "" 8 50 0) 2>&1 | head -n 1 > $resultfile 
}


main_menu(){
    $DIALOG --backtitle "$TITLE" \
	$CLEAR \
	--cancel-label " Exit " \
	--shadow \
	--title " Operations " \
        --menu "Message: $(cat $resultfile)" 18 60 11 \
	"etime"  " Exposure time ($ETIME) sec. "\
	"gain"  " Set gain factor ($GAIN) "\
	"filter" " Set current filter ($FILTER)" \
	"expose" " Exposure (not get) " \
	"darkseq" " Run dark sequence " \
	"flatseq" " Run flat sequence of dome" \
	"biasseq" " Run bias sequence " \
	"exposeseq" " Run exposure sequence " \
	"autodisp" " Toggle auto display ($AUTODISP) " \
	"delay"   " Delay between getting images ($DELAY) sec. "\
	"exit"   " Bye " 2> $tempfile

    case $? in
	1)  clear;
	    save_ustariconf
	    exit 0 ;;
	255)clear;exit 0 ;;
    esac

    case $(cat $tempfile) in
    etime)
	set_etime_menu 
	;;
    gain)
	set_gain_menu 
	;;
    filter)
	set_filter_menu
	;;
    expose) 
	(expose --percent $ETIME &)|\
	$DIALOG --backtitle "$TITLE" --title "Exposing" \
		--gauge "Exposure time: $ETIME sec." 6 40 0
	;;
    exposeseq)
	sequence_menu "expose" "" "Expose sequence"
	;;
    biasseq)
	sequence_menu "bias" "bias" "Bias sequence"
	;;
    darkseq)
	sequence_menu "dark $ETIME" "dark" "Dark sequence"
	;;
    flatseq)
	if [ ! -f $CONFFILE$LOCK_EXT ];then 
	    echo $MYPID > $CONFFILE$LOCK_EXT
	    (cat $CONFFILE| sed "s/^OBJECT/#ustari#OBJECT/"
	    echo -e "OBJECT = domeflat\t#Objet name") > $CONFFILE.tmp

	    mv -f $CONFFILE.tmp $CONFFILE

	    sequence_menu "flat $ETIME" "dflat$(echo $FILTER|tr A-Z a-z)" "Dome flat sequence"
	    (cat $CONFFILE|grep -v "^OBJECT"|\
	    sed "s/^#ustari#OBJECT/OBJECT/") > $CONFFILE.tmp
	    mv -f $CONFFILE.tmp $CONFFILE
	    rm -f $CONFFILE$LOCK_EXT
	else
	    echo "Configuration file is loked by pid $(cat $CONFFILE$LOCK_EXT)." > $resultfile
	fi
	;;
    autodisp)
	if [ "$AUTODISP" == "off" ];then
	    AUTODISP=on
	else
	    AUTODISP=off
	fi
	;;
    display)
	 #"display" " Display an image over ds9 " \
	;;
    delay)
	set_delay_menu
	;;
    exit)
	clear;
	save_ustariconf
	exit 0
	;;
    * ) 
	return;;
    esac
}


if [ ! -f $CONFFILE ]; then
    echo -e "\nConfiguration file not found. Run stari first\n\n"
    exit -1
fi

while [ 1 ];do
# FIXME: chequear que la camara este coriendo... 
    ETIME=$(etime)
    GAIN=$(gain)
    FILTER=$(filter)
    retval=$?
    #echo "Retval: $retval"
    case $retval in
    0) main_menu ;;    
    254) echo -e "\nCan not comunicated which Camera.\n"
	exit -2 ;;    
    127) echo "Error: command not found."
	exit -1;;
    *) echo "Error: Unknow."
	exit -1;;
    esac
done
